@@ -42,7 +42,7 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", depth=NULL
       Arguments$getInstanceOf(df, reqFileClass)
     });
   } else {
-    throw("Argument 'files' is of unknown type: ", mode(files)[1]);
+    throw("Argument 'files' is of unknown type: ", mode(files)[1L]);
   }
 
   # Arguments 'depth':
@@ -55,7 +55,7 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", depth=NULL
 
   # Arguments '...':
   args <- list(...);
-  if (length(args) > 0) {
+  if (length(args) > 0L) {
     if (is.element(.onUnknownArgs, c("error", "warning"))) {
       argsStr <- paste(names(args), collapse=", ");
       msg <- sprintf("Unknown arguments: %s", argsStr);
@@ -69,8 +69,8 @@ setConstructorS3("GenericDataFileSet", function(files=NULL, tags="*", depth=NULL
 
 
   this <- extend(Object(), c("GenericDataFileSet", uses("FullNameInterface")),
-    files = as.list(files),
     "cached:.fileSize" = NULL,
+    files = as.list(files),
     .depth = depth,
     .tags = NULL
   );
@@ -121,7 +121,7 @@ setMethodS3("as.character", "GenericDataFileSet", function(x, ...) {
   # To please R CMD check
   this <- x;
 
-  s <- sprintf("%s:", class(this)[1]);
+  s <- sprintf("%s:", class(this)[1L]);
 
   # Name and tags of file set
   s <- c(s, sprintf("Name: %s", getName(this)));
@@ -135,7 +135,7 @@ setMethodS3("as.character", "GenericDataFileSet", function(x, ...) {
 
   # Subdirectories(?)
   subdirs <- getSubdirs(this, default=NULL);
-  if (length(subdirs) > 0) {
+  if (length(subdirs) > 0L) {
     s <- c(s, sprintf("Subpath: %s", subdirs));
   }
 
@@ -149,11 +149,9 @@ setMethodS3("as.character", "GenericDataFileSet", function(x, ...) {
 
   # Pathname
   path <- getPath(this);
-  if (!is.null(path) && !is.na(path)) {
+  if (!is.na(path)) {
     pathR <- getRelativePath(path);
-    if (nchar(pathR) < nchar(path)) {
-      path <- pathR;
-    }
+    if (nchar(pathR) < nchar(path)) path <- pathR;
   }
   s <- c(s, paste("Path (to the first file): ", path, sep=""));
 
@@ -166,18 +164,19 @@ setMethodS3("as.character", "GenericDataFileSet", function(x, ...) {
 
   class(s) <- "GenericSummary";
   s;
-}, private=TRUE)
+}, protected=TRUE)
 
 
 
 
 setMethodS3("clearCache", "GenericDataFileSet", function(this, ...) {
   # Clear the cache of all files
-  lapply(this, clearCache);
+  files <- getFiles(this);
+  lapply(files, FUN=clearCache);
 
   # Then for this object
   NextMethod("clearCache");
-})
+}, protected=TRUE)
 
 
 
@@ -197,15 +196,15 @@ setMethodS3("clone", "GenericDataFileSet", function(this, clear=TRUE, ...) {
     clearCache(object);
 
   object;
-}, private=TRUE)
+}, protected=TRUE)
 
 
 
 setMethodS3("getFileClass", "GenericDataFileSet", function(static, ...) {
   # By default, infer the file class from the set class.
-  name <- class(static)[1];
+  name <- class(static)[1L];
   name <- gsub("Set$", "", name);
-  if (regexpr("File$", name) == -1) {
+  if (regexpr("File$", name) == -1L) {
     name <- paste(name, "File", sep="");
   }
   name;
@@ -260,7 +259,9 @@ setMethodS3("validate", "GenericDataFileSet", function(this, ...) {
 setMethodS3("getFileSize", "GenericDataFileSet", function(this, ..., force=FALSE) {
   fileSize <- this$.fileSize;
   if (force || is.null(fileSize)) {
-    fileSize <- sum(unlist(lapply(this, FUN=getFileSize), use.names=FALSE));
+    files <- getFiles(this);
+    fileSizes <- unlist(lapply(files, FUN=getFileSize), use.names=FALSE);
+    fileSize <- sum(fileSizes, na.rm=TRUE);
     this$.fileSize <-  fileSize;
   }
   fileSize;
@@ -294,12 +295,9 @@ setMethodS3("getFileSize", "GenericDataFileSet", function(this, ..., force=FALSE
 # }
 #*/###########################################################################
 setMethodS3("getPath", "GenericDataFileSet", function(this, ...) {
-  files <- getFiles(this);
-  if (length(files) == 0) {
-    naValue <- as.character(NA);
-    return(naValue);
-  }
-  getPath(files[[1]]);
+  # Find a file with a non-missing pathname
+  file <- getOneFile(this);
+  getPath(file, ...);
 })
 
 
@@ -347,7 +345,7 @@ setMethodS3("getSubdirs", "GenericDataFileSet", function(this, collapse="/", ...
   }
   dirs <- rev(dirs);
 
-  if (length(dirs) > 1 && !is.null(collapse)) {
+  if (length(dirs) > 1L && !is.null(collapse)) {
     dirs <- paste(dirs, collapse=collapse);
   }
 
@@ -394,45 +392,6 @@ setMethodS3("nbrOfFiles", "GenericDataFileSet", function(this, ...) {
 
 
 
-###########################################################################/**
-# @RdocMethod lapply
-#
-# @title "Applies a function to each of the data files"
-#
-# \description{
-#   @get "title".
-# }
-#
-# @synopsis
-#
-# \arguments{
-#  \item{...}{Not used.}
-# }
-#
-# \value{
-#   Returns a named @list where the names are the names of the date files
-#   according to @seemethod "getNames".
-# }
-#
-# @author
-#
-# \seealso{
-#   @seeclass
-# }
-#*/###########################################################################
-setMethodS3("lapply", "GenericDataFileSet", function(this, ...) {
-  res <- base::lapply(this$files, ...);
-  names(res) <- unlist(lapply(as.list(this), FUN=getFullName));
-  res;
-})
-
-setMethodS3("sapply", "GenericDataFileSet", function(this, ...) {
-  res <- sapply(this$files, ...);
-  names(res) <- unlist(lapply(as.list(this), FUN=getFullName));
-  res;
-})
-
-
 setMethodS3("reorder", "GenericDataFileSet", function(x, order, ...) {
   # To please R CMD check
   this <- x;
@@ -573,12 +532,14 @@ setMethodS3("sortBy", "GenericDataFileSet", function(this, by=c("lexicographic",
 # }
 #*/###########################################################################
 setMethodS3("getNames", "GenericDataFileSet", function(this, ...) {
-  res <- unlist(lapply(this, FUN=getName, ...));
+  files <- as.list(this, useNames=FALSE);
+  res <- unlist(lapply(files, FUN=getName, ...));
   unname(res);
 })
 
 setMethodS3("getFullNames", "GenericDataFileSet", function(this, ...) {
-  res <- unlist(lapply(this, FUN=getFullName, ...));
+  files <- as.list(this, useNames=FALSE);
+  res <- unlist(lapply(files, FUN=getFullName, ...));
   unname(res);
 })
 
@@ -625,6 +586,12 @@ setMethodS3("indexOf", "GenericDataFileSet", function(this, patterns=NULL, ...,
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Argument '...':
+  args <- list(...);
+  if (is.element("names", names(args))) {
+    throw("Unknown argument 'names' to indexOf() for GenericDataFileSet.");
+  }
+
   # Argument 'onMissing':
   onMissing <- match.arg(onMissing);
 
@@ -645,7 +612,7 @@ setMethodS3("indexOf", "GenericDataFileSet", function(this, patterns=NULL, ...,
   res <- lapply(patterns, FUN=function(pattern) {
     # First try matching a regular expression, then a fixed string.
     pattern0 <- pattern;
-    hasTags <- (regexpr(",", pattern, fixed=TRUE) != -1);
+    hasTags <- (regexpr(",", pattern, fixed=TRUE) != -1L);
     if (hasTags) {
       searchStrings <- fullnames;
     } else {
@@ -673,13 +640,13 @@ setMethodS3("indexOf", "GenericDataFileSet", function(this, patterns=NULL, ...,
     # - - - - - - - - - - - -
     # 2. Fixed string?
     # - - - - - - - - - - - -
-    if (length(idxs) == 0) {
+    if (length(idxs) == 0L) {
       pattern <- pattern0;
       idxs <- grep(pattern, searchStrings, fixed=TRUE);
     }
 
     # Nothing matched?
-    if (length(idxs) == 0) {
+    if (length(idxs) == 0L) {
       idxs <- naValue;
     }
 
@@ -731,7 +698,8 @@ setMethodS3("indexOf", "GenericDataFileSet", function(this, patterns=NULL, ...,
 # }
 #*/###########################################################################
 setMethodS3("getPathnames", "GenericDataFileSet", function(this, ...) {
-  unlist(lapply(this, FUN=getPathname))
+  files <- getFiles(this);
+  unlist(lapply(files, FUN=getPathname));
 })
 
 
@@ -781,7 +749,8 @@ setMethodS3("seq", "GenericDataFileSet", function(this, ...) {
 # @synopsis
 #
 # \arguments{
-#   \item{...}{Not used.}
+#   \item{...}{Arguments passed to @seemethod "getFiles".}
+#   \item{useNames}{If @TRUE, the list will be annotated with names.}
 # }
 #
 # \value{
@@ -797,14 +766,12 @@ setMethodS3("seq", "GenericDataFileSet", function(this, ...) {
 # @keyword IO
 # @keyword programming
 #*/###########################################################################
-setMethodS3("as.list", "GenericDataFileSet", function(x, ...) {
-  # To please R CMD check.
-  this <- x;
-
-  this$files;
+setMethodS3("as.list", "GenericDataFileSet", function(x, useNames=TRUE, ...) {
+  getFiles(x, ..., useNames=useNames);
 })
 
 
+
 ###########################################################################/**
 # @RdocMethod getFile
 #
@@ -835,8 +802,9 @@ setMethodS3("as.list", "GenericDataFileSet", function(x, ...) {
 # @keyword programming
 #*/###########################################################################
 setMethodS3("getFile", "GenericDataFileSet", function(this, idx, ...) {
-  if (length(idx) != 1)
+  if (length(idx) != 1L) {
     throw("Argument 'idx' must be a single index.");
+  }
   res <- this$files;
   n <- length(res);
   idx <- Arguments$getIndex(idx, max=n);
@@ -844,16 +812,77 @@ setMethodS3("getFile", "GenericDataFileSet", function(this, idx, ...) {
 })
 
 
-setMethodS3("getFiles", "GenericDataFileSet", function(this, idxs=NULL, ...) {
+setMethodS3("getFiles", "GenericDataFileSet", function(this, idxs=NULL, useNames=FALSE, ...) {
   res <- this$files;
-  if (is.null(idxs)) {
-  } else {
+
+  # Subset?
+  if (!is.null(idxs)) {
     n <- length(res);
     idxs <- Arguments$getIndices(idxs, max=n);
     res <- res[idxs];
   }
+
+  # Add names?
+  if (useNames) {
+    names(res) <- sapply(res, FUN=getFullName, ...);
+  }
+
   res;
-}, private=TRUE)
+}, protected=TRUE)
+
+
+setMethodS3("getOneFile", "GenericDataFileSet", function(this, default=NA, mustExist=is.null(default), ...) {
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Local functions
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  getDefault <- function() {
+    if (is.null(default)) return(NULL);
+    if (!is.object(default) && is.na(default)) {
+      className <- getFileClass(this);
+      clazz <- Class$forName(className);
+      default <- newInstance(clazz);
+    } else if (is.numeric(default)) {
+      default <- getFile(this, default);
+    }
+    default <- Arguments$getInstanceOf(default, "GenericDataFile");
+    default;
+  } # getDefault()
+
+
+  # Argument 'mustExist':
+  mustExist <- Arguments$getLogical(mustExist);
+
+
+  files <- getFiles(this);
+  I <- length(files);
+
+  # Nothing?
+  if (I == 0L) {
+    if (mustExist) {
+      throw("Cannot retrieve a file with a non-missing pathname. File set is empty.");
+    }
+    return(getDefault());
+  }
+
+
+  # Find first file with a non-missing pathname
+  for (ii in seq_len(I)) {
+    file <- files[[ii]];
+    pathname <- getPathname(file);
+    # Found?
+    if (!is.na(pathname)) {
+      return(file);
+    }
+  } # for (ii ...)
+
+
+  # Nothing found
+  if (mustExist) {
+    throw("Cannot retrieve a file with a non-missing pathname. File set contains no such files.");
+  }
+
+  return(getDefault());
+}) # getOneFile()
 
 
 
@@ -910,12 +939,12 @@ setMethodS3("appendFiles", "GenericDataFileSet", function(this, files, clone=TRU
     .fileClass <- Arguments$getCharacter(.fileClass);
   }
 
-  if (length(files) > 0) {
+  if (length(files) > 0L) {
     # Assert that all files are instances of the file class of this set.
     className <- .fileClass;
     isValid <- unlist(lapply(files, FUN=inherits, className));
     if (!all(isValid)) {
-      classNames <- sapply(files, FUN=function(x) class(x)[1]);
+      classNames <- sapply(files, FUN=function(x) class(x)[1L]);
       classNames <- classNames[!isValid];
       classNames <- unique(classNames);
       throw(sprintf("Argument 'files' contains non-%s objects: %s", 
@@ -923,12 +952,12 @@ setMethodS3("appendFiles", "GenericDataFileSet", function(this, files, clone=TRU
     }
 
     # Must inherit from the same class as the existing files?
-    if (.assertSameClass && length(this) > 0) {
-      aFile <- getFile(this, 1);
-      className <- class(aFile)[1];
+    if (.assertSameClass && length(this) > 0L) {
+      aFile <- getOneFile(this);
+      className <- class(aFile)[1L];
       isValid <- unlist(lapply(files, FUN=inherits, className));
       if (!all(isValid)) {
-        classNames <- sapply(files, FUN=function(x) class(x)[1]);
+        classNames <- sapply(files, FUN=function(x) class(x)[1L]);
         classNames <- classNames[!isValid];
         classNames <- unique(classNames);
         throw(sprintf("Argument 'files' contains non-%s objects (which is what the set already contains): %s", className, hpaste(classNames)));
@@ -945,7 +974,7 @@ setMethodS3("appendFiles", "GenericDataFileSet", function(this, files, clone=TRU
 
 
   verbose && enter(verbose, "Appending ", length(files), " files");
-  if (length(files) > 0) {
+  if (length(files) > 0L) {
     # Clone file objects?
     if (clone) {
       verbose && enter(verbose, "Cloning files");
@@ -1009,7 +1038,7 @@ setMethodS3("append", "GenericDataFileSet", function(x, values, ...) {
 
   # Argument 'other':
   if (inherits(other, "GenericDataFileSet")) {
-    other <- Arguments$getInstanceOf(other, class(this)[1]);
+    other <- Arguments$getInstanceOf(other, class(this)[1L]);
     files <- getFiles(other);
   } else {
     files <- other;
@@ -1066,12 +1095,12 @@ setMethodS3("extract", "GenericDataFileSet", function(this, files, ..., onMissin
     files <- Arguments$getIntegers(files, disallow="NaN");
 
     # Exclude indices?
-    if (any(files < 0, na.rm=TRUE)) {
-      incl <- files[files > 0];
-      if (length(incl) == 0) {
+    if (any(files < 0L, na.rm=TRUE)) {
+      incl <- files[files > 0L];
+      if (length(incl) == 0L) {
         incl <- seq_along(this);
       }
-      excl <- na.omit(files[files < 0]);
+      excl <- na.omit(files[files < 0L]);
       files <- setdiff(incl, -excl);
       rm(incl, excl);
     }
@@ -1093,15 +1122,15 @@ setMethodS3("extract", "GenericDataFileSet", function(this, files, ..., onMissin
   missing <- which(is.na(files));
 
   # Drop non-existing files?
-  if (length(missing) > 0 && onMissing == "drop") {
+  if (length(missing) > 0L && onMissing == "drop") {
     files <- files[is.finite(files)];
-    missing <- integer(0);
+    missing <- 0L;
   }
 
   # Check for duplicates?
   if (onDuplicates != "ignore") {
     dups <- which(is.finite(files) & duplicated(names(files)));
-    if (length(dups) > 0) {
+    if (length(dups) > 0L) {
       dupNames <- names(files)[head(dups)];
       dupNames <- paste(dupNames, collapse=", ");
       if (onDuplicates == "error") {
@@ -1118,8 +1147,8 @@ setMethodS3("extract", "GenericDataFileSet", function(this, files, ..., onMissin
   files <- this$files[files];
 
   # Any missing files?
-  if (length(missing) > 0) {
-    className <- class(this$files[[1]])[1];
+  if (length(missing) > 0L) {
+    className <- class(this$files[[1L]])[1L];
     if (is.null(className)) {
       className <- getFileClass(this);
     }
@@ -1204,11 +1233,11 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
   }
 
 
-  verbose && enter(verbose, "Defining an ", class(static)[1], " object from files");
+  verbose && enter(verbose, "Defining an ", class(static)[1L], " object from files");
   verbose && cat(verbose, "Path: ", path);
   verbose && cat(verbose, "Depth: ", depth);
   verbose && cat(verbose, "Pattern: ", pattern);
-  verbose && cat(verbose, "File class: ", class(dfStatic)[1]);
+  verbose && cat(verbose, "File class: ", class(dfStatic)[1L]);
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Create set of GenericDataFile objects from matching files
@@ -1218,7 +1247,7 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
   pathnames <- list.files(path=path, pattern=pattern, full.names=TRUE, 
                                    all.files=FALSE, recursive=recursive);
   verbose && printf(verbose, "Found %d files/directories.\n", length(pathnames));
-  if (length(pathnames) > 0) {
+  if (length(pathnames) > 0L) {
     # Keep only files
     keep <- sapply(pathnames, FUN=isFile);
     pathnames <- pathnames[keep];
@@ -1226,7 +1255,7 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
   verbose && printf(verbose, "Found %d files.\n", length(pathnames));
   verbose && exit(verbose);
 
-  if (length(pathnames) > 0) {
+  if (length(pathnames) > 0L) {
     # Sort files in lexicographic order
     pathnames <- sort(pathnames);
 
@@ -1244,7 +1273,7 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
         # that if the second file cannot be instanciated with the same
         # class as the first one, then the files are incompatible.
         # Note that 'df' might be of a subclass of 'dfStatic'.
-        clazz <- Class$forName(class(df)[1]);
+        clazz <- Class$forName(class(df)[1L]);
         dfStatic <- getStaticInstance(clazz);
       }
     }
@@ -1258,7 +1287,7 @@ setMethodS3("byPath", "GenericDataFileSet", function(static, path=NULL, pattern=
   if (inherits(static, "Class")) {
     className <- getName(static);
   } else {
-    className <- class(static)[1];
+    className <- class(static)[1L];
   }
   verbose && enter(verbose, "Allocating a new ", className, " instance");
   verbose && cat(verbose, "Arguments:");
@@ -1403,14 +1432,14 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Arguments 'name':
   name <- Arguments$getCharacter(name);
-  if (nchar(name) == 0) {
-    throw("A ", class(static)[1], " must have a non-empty name: ''");
+  if (nchar(name) == 0L) {
+    throw("A ", class(static)[1L], " must have a non-empty name: ''");
   }
   name <- Arguments$getFilename(name, .type="name");
 
   # Arguments 'tags':
   if (!is.null(tags)) {
-    tags <- sapply(tags, Arguments$getFilename, .type="name", .name="tags");
+    tags <- sapply(tags, FUN=Arguments$getFilename, .type="name", .name="tags");
   }
 
   # Arguments 'paths':
@@ -1451,7 +1480,7 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
   verbose && cat(verbose, "Possible search paths after expansion:");
   verbose && print(verbose, paths);
 
-  if (length(paths) == 0) {
+  if (length(paths) == 0L) {
     if (mustExist) {
       throw("No such root path directories: ", paste(paths, collapse=", "));
     }
@@ -1470,7 +1499,7 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
   rootPaths <- sapply(paths, FUN=function(path) {
     Arguments$getReadablePath(path, mustExist=FALSE);
   });
-  if (length(rootPaths) == 0) {
+  if (length(rootPaths) == 0L) {
     if (mustExist) {
       throw("None of the root path directories exist: ", 
                                            paste(paths, collapse=", "));
@@ -1508,11 +1537,11 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
 
 
   paths <- NULL;
-  if (length(dataSetPaths) > 0) {
+  if (length(dataSetPaths) > 0L) {
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     # Identify existing subdirectories
     # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-    if (length(subdirs) >= 1) {
+    if (length(subdirs) >= 1L) {
       verbose && enter(verbose, "Search subdirectories");
       verbose && print(verbose, subdirs);
 
@@ -1524,7 +1553,7 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
         if (identical(dir, "*"))
           dir <- ":.*:";
         pattern <- "^:([^:]*):$";
-        isSmart <- (regexpr(pattern, dir) != -1);
+        isSmart <- (regexpr(pattern, dir) != -1L);
         if (isSmart) {
           verbose && enter(verbose, "Processing \"smart\" path");
 
@@ -1535,14 +1564,14 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
           pathsKK <- sapply(dataSetPaths, FUN=function(path) {
             # List all directories and files
             dirsT <- list.files(path=path, pattern=pattern, full.names=TRUE);
-            if (length(dirsT) == 0)
+            if (length(dirsT) == 0L)
               return(NULL);
             # Keep only directories
             dirsT <- sapply(dirsT, FUN=function(path) {
               Arguments$getReadablePath(path, mustExist=FALSE);
             });
             dirsT <- dirsT[sapply(dirsT, FUN=isDirectory)];
-            if (length(dirsT) == 0)
+            if (length(dirsT) == 0L)
               return(NULL);
             # Work only with the directory names
             dirsT <- basename(dirsT);
@@ -1550,7 +1579,7 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
             # Keep only the first match
             # TO DO: Find a more powerful set of selecting directories
             # /HB 2009-02-11
-            dir <- dirsT[1];
+            dir <- dirsT[1L];
 
             file.path(path, dir);
           });
@@ -1583,19 +1612,19 @@ setMethodS3("findByName", "GenericDataFileSet", function(static, name, tags=NULL
       verbose && exit(verbose);
     } else {
       paths <- dataSetPaths;
-    } # if (length(subdirs) >= 1)
+    } # if (length(subdirs) >= 1L)
 
 
-    if (length(paths) > 1) {
+    if (length(paths) > 1L) {
       if (firstOnly) {
         warning("Found duplicated data set: ", paste(paths, collapse=", "));
-        paths <- paths[1];
+        paths <- paths[1L];
         verbose && cat(verbose, "Dropped all but the first path.");
       }
     }
-  } # if (length(dataSetPaths) > 0)
+  } # if (length(dataSetPaths) > 0L)
   
-  if (length(paths) == 0) {
+  if (length(paths) == 0L) {
     paths <- NULL;
 
     if (mustExist) {
@@ -1664,12 +1693,12 @@ setMethodS3("byName", "GenericDataFileSet", function(static, name, tags=NULL, su
     on.exit(popState(verbose));
   }
 
-  verbose && enter(verbose, sprintf("Setting up %s by its name", class(static)[1]));
+  verbose && enter(verbose, sprintf("Setting up %s by its name", class(static)[1L]));
   verbose && cat(verbose, "Name: ", name);
   verbose && cat(verbose, "Tags: ", paste(tags, collapse=","));
 
   # Record the "depth"/"subdirs".
-  if (length(subdirs) > 0) {
+  if (length(subdirs) > 0L) {
     subdirsT <- unlist(strsplit(subdirs, split="/\\\\"), use.names=FALSE);
     depth <- length(subdirsT);
   } else {
@@ -1749,7 +1778,7 @@ setMethodS3("equals", "GenericDataFileSet", function(this, other, ..., verbose=F
 
   if (!inherits(other, "GenericDataFileSet")) {
     msg <- sprintf("The 'other' is not a GenericDataFileSet: %s",
-                                                 class(other)[1]);
+                                                 class(other)[1L]);
     attr(notEqual, "reason") <- msg;
     return(notEqual);
   }
@@ -1770,15 +1799,13 @@ setMethodS3("equals", "GenericDataFileSet", function(this, other, ..., verbose=F
 
   for (kk in seq_along(this)) {
     verbose && enter(verbose, sprintf("File #%d of %d", kk, nbrOfFiles));
-
-    df1 <- getFile(this, 1);
-    df2 <- getFile(other, 1);
+    df1 <- getFile(this, kk);
+    df2 <- getFile(other, kk);
     eqls <- equals(df1, df2, ...);
     if (!eqls) {
       verbose && cat(verbose, "Not equal");
       return(eqls);
     }
-
     verbose && exit(verbose);
   }
 
@@ -1851,7 +1878,7 @@ setMethodS3("getDefaultFullName", "GenericDataFileSet", function(this, parent=ge
   }
 
   if (!is.null(parent)) {
-    while (parent > 0) {
+    while (parent > 0L) {
       # path/to/<fullname>/<something>
       path <- dirname(path);
       parent <- parent - 1;
@@ -1880,42 +1907,54 @@ setMethodS3("updateFullNames", "GenericDataFileSet", function(this, ...) {
 
 
 setMethodS3("clearFullNamesTranslator", "GenericDataFileSet", function(this, ...) {
-  sapply(this, clearFullNameTranslator, ...);
+  files <- as.list(this, useNames=FALSE);
+  lapply(files, FUN=clearFullNameTranslator, ...);
   invisible(this);
 }, protected=TRUE)
 
+setMethodS3("resetFullNames", "GenericDataFileSet", function(this, ...) {
+  clearFullNamesTranslator(this, ...);
+}, protected=TRUE)
+
+
 setMethodS3("appendFullNamesTranslatorByNULL", "GenericDataFileSet", function(this, ...) {
-  sapply(this, appendFullNameTranslatorByNULL, NULL, ...);
+  files <- as.list(this, useNames=FALSE);
+  sapply(files, FUN=appendFullNameTranslatorByNULL, NULL, ...);
   invisible(this);
 }, protected=TRUE)
 
 
 setMethodS3("appendFullNamesTranslatorByfunction", "GenericDataFileSet", function(this, fcn, ...) {
-  sapply(this, appendFullNameTranslatorByfunction, fcn, ...);
+  files <- as.list(this, useNames=FALSE);
+  sapply(files, FUN=appendFullNameTranslatorByfunction, fcn, ...);
   invisible(this);
 }, protected=TRUE)
 
 
 setMethodS3("appendFullNamesTranslatorBydata.frame", "GenericDataFileSet", function(this, fcn, ...) {
-  sapply(this, appendFullNameTranslatorBydata.frame, fcn, ...);
+  files <- as.list(this, useNames=FALSE);
+  sapply(files, FUN=appendFullNameTranslatorBydata.frame, fcn, ...);
   invisible(this);
 }, protected=TRUE)
 
 
 setMethodS3("appendFullNamesTranslatorByTabularTextFile", "GenericDataFileSet", function(this, fcn, ...) {
-  sapply(this, appendFullNameTranslatorByTabularTextFile, fcn, ...);
+  files <- as.list(this, useNames=FALSE);
+  sapply(files, FUN=appendFullNameTranslatorByTabularTextFile, fcn, ...);
   invisible(this);
 }, protected=TRUE)
 
 
 setMethodS3("appendFullNamesTranslatorByTabularTextFileSet", "GenericDataFileSet", function(this, fcn, ...) {
-  sapply(this, appendFullNameTranslatorByTabularTextFileSet, fcn, ...);
+  files <- as.list(this, useNames=FALSE);
+  sapply(files, FUN=appendFullNameTranslatorByTabularTextFileSet, fcn, ...);
   invisible(this); 
 }, protected=TRUE)
 
 
 setMethodS3("appendFullNamesTranslatorBylist", "GenericDataFileSet", function(this, fcn, ...) {
-  sapply(this, appendFullNameTranslatorBylist, fcn, ...);
+  files <- as.list(this, useNames=FALSE);
+  sapply(files, FUN=appendFullNameTranslatorBylist, fcn, ...);
   invisible(this);
 }, protected=TRUE)
 
@@ -1929,13 +1968,13 @@ setMethodS3("appendFullNamesTranslator", "GenericDataFileSet", function(this, by
   keep <- sapply(methodNames, FUN=exists, mode="function");
   methodNames <- methodNames[keep];
 
-  if (length(methodNames) > 0) {
-    methodName <- methodNames[1];
+  if (length(methodNames) > 0L) {
+    methodName <- methodNames[1L];
     fcn <- get(methodName, mode="function");
     res <- fcn(this, by, ...);
   } else {
     # ...otherwise, apply the fullname translator to each file
-    res <- sapply(this, appendFullNameTranslator, by, ...);
+    res <- sapply(this, FUN=appendFullNameTranslator, by, ...);
   }
 
   # Allow the object to update itself according to these new rules.
@@ -1954,6 +1993,29 @@ setMethodS3("setFullNamesTranslator", "GenericDataFileSet", function(this, ...)
 
 ############################################################################
 # HISTORY:
+# 2012-12-20
+# o ROBUSTNESS: Now indexOf() for GenericDataFileSet throws an exception
+#   if use tries to pass an argument 'names'.
+# 2012-12-09
+# o Added [[() for GenericDataFileSet.
+# o Now getOneFile() returns as soon as possible.  Before it was querying
+#   the pathnames of all files before making a decision.
+# 2012-12-06
+# o BUG FIX: equals() for GenericDataFileSet would only compare the first
+#   GenericDataFile in each set.
+# o Added getOneFile() for GenericDataFileSet, which returns the first
+#   GenericDataFile with a non-missing pathname.
+# 2012-11-30
+# o Added resetFullNames() for GenericDataFileSet, which is an alias
+#   for clearFullNamesTranslator().
+# 2012-11-29
+# o CLEANUP: Removed lapply() and sapply() for GenericDataSet because
+#   the corresponding functions in the 'base' package utilizes 
+#   as.list().
+# o Added argument 'useNames' to as.list() for GenericDataFileSet.
+# o Added argument 'useNames' to getFiles() for GenericDataFileSet.
+# o Now lapply() and sapply() for GenericDataFileSet pass '...' also
+#   to getFullName() for each file, e.g. translate=FALSE.
 # 2012-11-13
 # o CLEANUP: Now clearCache() for GenericDataFileSet relies on ditto
 #   of Object to clear all cached fields (=with field modifier "cached").
